import { getFieldDef } from 'graphql/execution/execute';
import { isObjectType, isListType, isNonNullType } from 'graphql';
import { pathOr } from 'ramda';
import chalk from 'chalk';

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function _inheritsLoose(subClass, superClass) {
  subClass.prototype = Object.create(superClass.prototype);
  subClass.prototype.constructor = subClass;
  subClass.__proto__ = superClass;
}

function _getPrototypeOf(o) {
  _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) {
    return o.__proto__ || Object.getPrototypeOf(o);
  };
  return _getPrototypeOf(o);
}

function _setPrototypeOf(o, p) {
  _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) {
    o.__proto__ = p;
    return o;
  };

  return _setPrototypeOf(o, p);
}

function isNativeReflectConstruct() {
  if (typeof Reflect === "undefined" || !Reflect.construct) return false;
  if (Reflect.construct.sham) return false;
  if (typeof Proxy === "function") return true;

  try {
    Date.prototype.toString.call(Reflect.construct(Date, [], function () {}));
    return true;
  } catch (e) {
    return false;
  }
}

function _construct(Parent, args, Class) {
  if (isNativeReflectConstruct()) {
    _construct = Reflect.construct;
  } else {
    _construct = function _construct(Parent, args, Class) {
      var a = [null];
      a.push.apply(a, args);
      var Constructor = Function.bind.apply(Parent, a);
      var instance = new Constructor();
      if (Class) _setPrototypeOf(instance, Class.prototype);
      return instance;
    };
  }

  return _construct.apply(null, arguments);
}

function _isNativeFunction(fn) {
  return Function.toString.call(fn).indexOf("[native code]") !== -1;
}

function _wrapNativeSuper(Class) {
  var _cache = typeof Map === "function" ? new Map() : undefined;

  _wrapNativeSuper = function _wrapNativeSuper(Class) {
    if (Class === null || !_isNativeFunction(Class)) return Class;

    if (typeof Class !== "function") {
      throw new TypeError("Super expression must either be null or a function");
    }

    if (typeof _cache !== "undefined") {
      if (_cache.has(Class)) return _cache.get(Class);

      _cache.set(Class, Wrapper);
    }

    function Wrapper() {
      return _construct(Class, arguments, _getPrototypeOf(this).constructor);
    }

    Wrapper.prototype = Object.create(Class.prototype, {
      constructor: {
        value: Wrapper,
        enumerable: false,
        writable: true,
        configurable: true
      }
    });
    return _setPrototypeOf(Wrapper, Class);
  };

  return _wrapNativeSuper(Class);
}

function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;

  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }

  return target;
}

// A type of promise-like that resolves synchronously and supports only one observer

const _iteratorSymbol = /*#__PURE__*/ typeof Symbol !== "undefined" ? (Symbol.iterator || (Symbol.iterator = Symbol("Symbol.iterator"))) : "@@iterator";

const _asyncIteratorSymbol = /*#__PURE__*/ typeof Symbol !== "undefined" ? (Symbol.asyncIterator || (Symbol.asyncIterator = Symbol("Symbol.asyncIterator"))) : "@@asyncIterator";

// Asynchronously call a function and send errors to recovery continuation
function _catch(body, recover) {
	try {
		var result = body();
	} catch(e) {
		return recover(e);
	}
	if (result && result.then) {
		return result.then(void 0, recover);
	}
	return result;
}

var IGNORED_FIELD_NAMES = ['__typename'];
var FIELD_PARAM_PREFIX = 'field_';

var FieldMissingError =
/*#__PURE__*/
function (_Error) {
  _inheritsLoose(FieldMissingError, _Error);

  function FieldMissingError(typeName, fieldName) {
    return _Error.call(this, "Invalid state: field \"" + fieldName + "\" does not exist on type \"" + typeName + "\"") || this;
  }

  return FieldMissingError;
}(_wrapNativeSuper(Error));

var getArgumentsPlusDefaults = function getArgumentsPlusDefaults(parentTypeName, field, schema, variables) {
  var schemaType = schema.getType(parentTypeName);

  if (!schemaType || !isObjectType(schemaType)) {
    throw new Error("Invalid state: Unknown or non-object type name \"" + parentTypeName + "\" (type: " + schemaType + ")");
  }

  var schemaField = schemaType.getFields()[field.name.value];

  if (!schemaField) {
    throw new FieldMissingError(schemaType.name, field.name.value);
  }

  var defaults = schemaField.args.reduce(function (argMap, arg) {
    var _extends2;

    return arg.defaultValue !== undefined ? _extends({}, argMap, (_extends2 = {}, _extends2[arg.name] = arg.defaultValue, _extends2)) : argMap;
  }, {});
  return _extends({}, defaults, {}, argFieldsToValues({}, field.arguments || [], variables));
};
var argFieldsToValues = function argFieldsToValues(providedValues, fields, variables) {
  return fields.reduce(function (acc, fieldNode) {
    acc[fieldNode.name.value] = valueNodeToValue(fieldNode.value, variables);
    return acc;
  }, providedValues);
};
var valueNodeToValue = function valueNodeToValue(valueNode, variables) {
  if (valueNode.kind === 'Variable') {
    return variables[valueNode.name.value];
  } else if (valueNode.kind === 'NullValue') {
    return null;
  } else if (valueNode.kind === 'ObjectValue') {
    return argFieldsToValues({}, valueNode.fields, variables);
  } else if (valueNode.kind === 'ListValue') {
    return valueNode.values.map(function (value) {
      return valueNodeToValue(value, variables);
    });
  } else if (valueNode.kind === 'IntValue') {
    return parseInt(valueNode.value, 10);
  } else if (valueNode.kind === 'FloatValue') {
    return parseFloat(valueNode.value);
  } else {
    return valueNode.value;
  }
};
var isListOrWrappedListType = function isListOrWrappedListType(type) {
  if (isListType(type)) {
    return true;
  }

  if (isNonNullType(type)) {
    return isListOrWrappedListType(type.ofType);
  }

  return false;
};
var getNameOrAlias = function getNameOrAlias(field) {
  return field.alias ? field.alias.value : field.name.value;
};
var extractObjectType = function extractObjectType(type) {
  if (isObjectType(type)) {
    return type;
  } // TODO: interface / union ?


  if (isNonNullType(type) || isListType(type)) {
    return extractObjectType(type.ofType);
  }

  return null;
};
/**
 * Converts a path from `info` into a field path, skipping over
 * array indices (since they are not represented in the schema
 * field selection paths)
 */

function getFieldPath(info) {
  var path = [];
  var pathLink = info.path;

  while (pathLink) {
    if (typeof pathLink.key === 'string') {
      path.unshift(pathLink.key);
    }

    pathLink = pathLink.prev;
  }

  return path;
}

var getDirectiveArgs = function getDirectiveArgs(directive, variableValues) {
  if (!directive || !directive.arguments) {
    return {};
  }

  return directive.arguments.reduce(function (args, arg) {
    var _extends2;

    return _extends({}, args, (_extends2 = {}, _extends2[arg.name.value] = valueNodeToValue(arg.value, variableValues), _extends2));
  }, {});
};
var getFieldDirectives = function getFieldDirectives(parentType, fieldName) {
  if (!isObjectType(parentType)) {
    throw new Error("Cannot traverse field \"" + fieldName + "\" on non-object type \"" + parentType + "\"");
  }

  var field = parentType.getFields()[fieldName];

  if (!field) {
    throw new Error("Field \"" + fieldName + "\" does not exist on type \"" + parentType.name + "\"");
  }

  if (!field.astNode) {
    throw new Error("Field \"" + parentType.name + "." + field + "\" doesn't have an AST node");
  }

  return field.astNode.directives || [];
};

var lines = function lines(strings, joiner) {
  if (joiner === void 0) {
    joiner = '\n';
  }

  return strings.filter(Boolean).join(joiner);
};
var indent = function indent(line) {
  return line.split('\n').map(function (l) {
    return "  " + l;
  }).join('\n');
};

var document = {
  name: 'document',
  build: function build(_ref) {
    var fieldName = _ref.fieldName,
        directiveArgs = _ref.directiveArgs,
        returnsList = _ref.returnsList;

    if (returnsList) {
      return "FOR " + fieldName + " IN " + directiveArgs.collection;
    } // for a singular field without a key arg, we just take the first
    // item out of the list


    if (!returnsList && !directiveArgs.key) {
      return lines(["FIRST(", indent("FOR " + fieldName + "_i IN " + directiveArgs.collection), indent("LIMIT 1"), indent("RETURN " + fieldName + "_i"), ")"]);
    } // possibly dangerous? a check to see if this is meant to be an interpolation
    // or if we need to treat it as a literal string


    var key = directiveArgs.key.startsWith('$') ? directiveArgs.key : "\"" + directiveArgs.key + "\"";
    return "LET " + fieldName + " = DOCUMENT(" + directiveArgs.collection + ", " + key + ")";
  }
};

var node = {
  name: 'node',
  build: function build(_ref) {
    var directiveArgs = _ref.directiveArgs,
        returnsList = _ref.returnsList,
        fieldName = _ref.fieldName,
        parentName = _ref.parentName;

    if (returnsList) {
      return "FOR " + fieldName + " IN " + directiveArgs.direction + " " + parentName + " " + directiveArgs.edgeCollection;
    }

    return lines(["LET " + fieldName + " = FIRST(", indent("FOR " + fieldName + "_i IN " + directiveArgs.direction + " " + parentName + " " + directiveArgs.edgeCollection), indent('LIMIT 1'), indent("RETURN " + fieldName + "_i"), ')']);
  }
};

var filter = {
  name: 'filter',
  build: function build(_ref) {
    var directiveArgs = _ref.directiveArgs;
    return "FILTER " + directiveArgs.statement;
  }
};

var sort = {
  name: 'sort',
  build: function build(_ref) {
    var directiveArgs = _ref.directiveArgs;
    // possibly dangerous? a check to see if this is meant to be an interpolation
    // or if we need to treat it as a literal string.
    var property = directiveArgs.property.startsWith('$') ? directiveArgs.property : "\"" + directiveArgs.property + "\"";
    return "SORT $field[" + property + "]" + (directiveArgs.order ? " " + directiveArgs.order : '');
  }
};

var limit = {
  name: 'limit',
  build: function build(_ref) {
    var directiveArgs = _ref.directiveArgs;
    return "LIMIT " + (directiveArgs.skip !== undefined ? directiveArgs.skip + ", " : '') + directiveArgs.count;
  }
};

var edge = {
  name: 'edge',
  build: function build(_ref) {
    var directiveArgs = _ref.directiveArgs,
        returnsList = _ref.returnsList,
        fieldName = _ref.fieldName,
        parentName = _ref.parentName;

    if (returnsList) {
      return "FOR " + fieldName + "_node, " + fieldName + " IN " + directiveArgs.direction + " " + parentName + " " + directiveArgs.edgeCollection;
    }

    return lines(["LET " + fieldName + " = FIRST(", indent("FOR " + fieldName + "_i_node, " + fieldName + "_i IN " + directiveArgs.direction + " " + parentName + " " + directiveArgs.edgeCollection), indent('LIMIT 1'), indent("RETURN " + fieldName + "_i"), ')']);
  }
};

var edgeNode = {
  name: 'edgeNode',
  build: function build(_ref) {
    var parentName = _ref.parentName;
    // this is assuming we are in the scope of a parent @edge subquery
    return "LET $field = " + parentName + "_node";
  }
};

var aql = {
  name: 'aql',
  build: function build(_ref) {
    var directiveArgs = _ref.directiveArgs;
    return "LET $field = " + directiveArgs.expression;
  }
};

var subquery = {
  name: 'subquery',
  build: function build(_ref) {
    var directiveArgs = _ref.directiveArgs;
    return directiveArgs.query;
  }
};

var key = {
  name: 'key',
  build: function build() {
    return "LET $field = $parent._key";
  }
};

var defaultPlugins = {
  document: document,
  node: node,
  filter: filter,
  sort: sort,
  limit: limit,
  edge: edge,
  edgeNode: edgeNode,
  aql: aql,
  subquery: subquery,
  key: key
};

var createFieldArgGetter = function createFieldArgGetter(fieldName) {
  return function (argPath) {
    return argPath.replace('$args.', "@" + FIELD_PARAM_PREFIX + fieldName + ".args.");
  };
};
/**
 * Creates a function which replaces all "$arg.foo.bar" with the real
 * argument string
 */

var createArgReplacer = function createArgReplacer(argGetter) {
  return function (str) {
    var argMatcher = /\$args(\.\w[\w\d]+)*/;
    var result;
    var modifiedStr = '' + str;

    while ((result = argMatcher.exec(modifiedStr)) !== null) {
      var text = result[0];
      var index = result.index;
      modifiedStr = spliceString(modifiedStr, index, text, argGetter(text));
    }

    return modifiedStr;
  };
};
/**
 * Creates a function which replaces all "$field" with the actual field name
 */

var createFieldReplacer = function createFieldReplacer(fieldName) {
  return function (text) {
    return replaceAll(text, '$field', fieldName);
  };
};

var createParentReplacer = function createParentReplacer(parentName) {
  return function (text) {
    return replaceAll(text, '$parent', parentName);
  };
};

var createContextReplacer = function createContextReplacer() {
  return function (text) {
    return replaceAll(text, '$context', '@context');
  };
};

var replaceAll = function replaceAll(text, original, replacement) {
  var modifiedText = '' + text;
  var index;

  while ((index = modifiedText.indexOf(original)) >= 0) {
    modifiedText = spliceString(modifiedText, index, original, replacement);
  }

  return modifiedText;
};

var spliceString = function spliceString(text, index, original, replacement) {
  return text.slice(0, index) + replacement + text.slice(index + original.length);
};

var createAllReplacer = function createAllReplacer(_ref) {
  var fieldName = _ref.fieldName,
      parentName = _ref.parentName;
  var argReplacer = createArgReplacer(createFieldArgGetter(fieldName));
  var fieldReplacer = createFieldReplacer(fieldName);
  var parentReplacer = createParentReplacer(parentName);
  var contextReplacer = createContextReplacer();
  return function (text) {
    return contextReplacer(parentReplacer(fieldReplacer(argReplacer(text))));
  };
};

var buildQuery = function buildQuery(_ref) {
  var query = _ref.query,
      fieldName = _ref.fieldName,
      parentName = _ref.parentName;
  var statements = query.plugins.map(function (_ref2) {
    var directiveArgs = _ref2.directiveArgs,
        plugin = _ref2.plugin;
    var fieldArgs = query.params.args || {};
    var interpolate = createAllReplacer({
      fieldName: fieldName,
      parentName: parentName
    });
    return interpolate(plugin.build({
      fieldName: fieldName,
      parentName: parentName,
      fieldArgs: fieldArgs,
      directiveArgs: directiveArgs,
      returnsList: query.returnsList
    }));
  });
  return lines([].concat(statements, [buildReturnProjection({
    query: query,
    fieldName: fieldName
  })]));
};

var buildReturnProjection = function buildReturnProjection(_ref3) {
  var query = _ref3.query,
      fieldName = _ref3.fieldName;

  if (!query.fieldNames.length) {
    return "RETURN " + fieldName;
  }

  var scalarFields = query.fieldNames.filter(function (name) {
    return !query.fieldQueries[name];
  });
  var nonScalarFields = query.fieldNames.filter(function (name) {
    return query.fieldQueries[name];
  });
  return lines(["RETURN {", lines(scalarFields.map(function (name) {
    return name + ": " + fieldName + "." + name;
  }).map(indent), ',\n'), lines(nonScalarFields.map(function (name) {
    var fieldQuery = query.fieldQueries[name];
    var subQueryString = buildQuery({
      query: fieldQuery,
      fieldName: joinFieldNames(fieldName, name),
      parentName: fieldName
    });
    return lines([name + ": (", indent(subQueryString), ")"]);
  }).map(indent), ',\n'), "}"]);
};

var joinFieldNames = function joinFieldNames(baseName, name) {
  return baseName + "_" + name;
};

var tag = '[graphql-arangodb] ';
var log = function log(_ref) {
  var title = _ref.title,
      level = _ref.level,
      details = _ref.details;

  if (level === 'verbose' && process.env.NODE_ENV !== "production" && process.env.DEBUG) {
    console.debug([chalk.yellow(tag + title)].concat(details.map(function (str) {
      return chalk.gray(str);
    })).join('\n'));
  } else if (level === 'debug' && process.env.DEBUG) {
    console.debug([chalk.cyan(tag + title)].concat(details.map(function (str) {
      return chalk.gray(str);
    })).join('\n'));
  } else if (level === 'error') {
    console.error([tag + title].concat(details).join('\n'));
  } else if (level === 'info') {
    console.info([chalk.blue(tag + title)].concat(details.map(function (str) {
      return chalk.gray(str);
    })).join('\n'));
  }
};

/**
 * recursively flattens and builds a set of arg object variables for a query
 * and all its sub-queries
 */

var buildPrefixedFieldArgVariables = function buildPrefixedFieldArgVariables(_ref) {
  var _extends2;

  var fieldName = _ref.fieldName,
      query = _ref.query;
  return _extends((_extends2 = {}, _extends2["" + FIELD_PARAM_PREFIX + fieldName] = {
    args: query.params.args
  }, _extends2), query.fieldNames.filter(function (childFieldName) {
    return !!query.fieldQueries[childFieldName];
  }).reduce(function (args, childFieldName) {
    return _extends({}, args, {}, buildPrefixedFieldArgVariables({
      fieldName: fieldName + '_' + childFieldName,
      query: query.fieldQueries[childFieldName]
    }));
  }, {}));
};

var buildPrefixedVariables = function buildPrefixedVariables(_ref2) {
  var fieldName = _ref2.fieldName,
      query = _ref2.query,
      parent = _ref2.parent,
      contextValues = _ref2.contextValues;
  return _extends({
    // passing the parent as a variable lets us cross the graphql -> graphdb boundary
    // and give queries access to their parent objects from our GraphQL context
    parent: parent,
    // the user may supply values in their context which they always want passed to queries
    context: contextValues
  }, buildPrefixedFieldArgVariables({
    fieldName: fieldName,
    query: query
  }));
};

var runQuery = function runQuery(_ref) {
  var returnsList = _ref.returnsList,
      query = _ref.query,
      bindVars = _ref.bindVars,
      db = _ref.db;

  try {
    return Promise.resolve(db.query({
      query: query,
      bindVars: bindVars
    })).then(function (queryResult) {
      return Promise.resolve(queryResult.all()).then(function (allResults) {
        return returnsList ? allResults : allResults[0];
      });
    });
  } catch (e) {
    return Promise.reject(e);
  }
};

var createResolver = function createResolver(_ref) {
  var _ref$plugins = _ref.plugins,
      plugins = _ref$plugins === void 0 ? defaultPlugins : _ref$plugins,
      _ref$argumentResolver = _ref.argumentResolvers,
      argumentResolvers = _ref$argumentResolver === void 0 ? {} : _ref$argumentResolver,
      db = _ref.db,
      _ref$contextDbKey = _ref.contextDbKey,
      contextDbKey = _ref$contextDbKey === void 0 ? 'arangoDb' : _ref$contextDbKey,
      _ref$contextKey = _ref.contextKey,
      contextKey = _ref$contextKey === void 0 ? 'arangoContext' : _ref$contextKey;
  return function (parent, args, context, info) {
    try {
      var resolvedDb = db || context[contextDbKey];

      if (!resolvedDb) {
        throw new Error("Either a valid ArangoDB Database instance must be supplied on the \"" + contextDbKey + "\" property of the context, or you must create your own resolver using createResolver from graphql-arangodb");
      }

      var path = getFieldPath(info);
      var query = extractQueriesFromField({
        info: info,
        parentQuery: undefined,
        parentType: info.parentType,
        field: info.fieldNodes[0],
        path: path,
        plugins: plugins,
        argumentResolvers: argumentResolvers
      });

      if (!query) {
        return Promise.resolve(null);
      }

      return Promise.resolve(_catch(function () {
        var queryString = buildQuery({
          query: query,
          fieldName: info.fieldName,
          parentName: ''
        });
        var bindVars = buildPrefixedVariables({
          fieldName: info.fieldName,
          query: query,
          parent: parent,
          contextValues: context[contextKey]
        });
        log({
          title: "Running query",
          level: 'info',
          details: [queryString, JSON.stringify(bindVars)]
        });
        return Promise.resolve(runQuery({
          returnsList: query.returnsList,
          query: queryString,
          bindVars: bindVars,
          db: resolvedDb
        })).then(function (data) {
          log({
            title: "Query response data",
            level: 'debug',
            details: [JSON.stringify(data)]
          });
          return data[info.fieldName];
        });
      }, function (err) {
        log({
          title: "Query execution error",
          level: 'error',
          details: [err.toString(), err.stack]
        });
        throw err;
      }));
    } catch (e) {
      return Promise.reject(e);
    }
  };
};
var resolver =
/*#__PURE__*/
createResolver({});
var extractQueriesFromField = function extractQueriesFromField(_ref2) {
  var info = _ref2.info,
      parentQuery = _ref2.parentQuery,
      parentType = _ref2.parentType,
      field = _ref2.field,
      path = _ref2.path,
      plugins = _ref2.plugins,
      argumentResolvers = _ref2.argumentResolvers;
  var fieldName = field.name.value;

  if (IGNORED_FIELD_NAMES.includes(fieldName)) {
    return null;
  }

  if (parentQuery) {
    parentQuery.fieldNames.push(fieldName);
  }

  var schemaFieldDef = getFieldDef(info.schema, parentType, fieldName);

  if (!schemaFieldDef) {
    throw new Error("Invalid state: there's no field definition for field \"" + fieldName + "\" on type \"" + parentType.name + "\"");
  }

  var directives = getFieldDirectives(parentType, fieldName);
  var pluginInstances = directives.map(function (directive) {
    var matchingPlugin = plugins[directive.name.value];

    if (!matchingPlugin) {
      return null;
    }

    return {
      plugin: matchingPlugin,
      directiveArgs: getDirectiveArgs(directive, info.variableValues)
    };
  }).filter(Boolean);

  if (!pluginInstances.length) {
    return null;
  }

  var argValues = getArgumentsPlusDefaults(parentType.name, field, info.schema, info.variableValues);
  var paramNames = [];
  var params = {};

  if (Object.keys(argValues).length) {
    paramNames.push('args'); // process via arg resolver if it exists

    var argResolver = pathOr(function (a) {
      return a;
    }, path, argumentResolvers);
    var resolvedArgs = argResolver(argValues);
    params.args = resolvedArgs;
  }

  var baseQuery = {
    returnsList: isListOrWrappedListType(schemaFieldDef.type),
    plugins: pluginInstances,
    paramNames: paramNames,
    params: params,
    fieldNames: [],
    fieldQueries: {}
  };

  if (!field.selectionSet) {
    return baseQuery;
  }

  var currentTypeAsObjectType = extractObjectType(schemaFieldDef.type);

  if (!currentTypeAsObjectType) {
    return baseQuery;
  }

  baseQuery.fieldQueries = extractQueriesFromSelectionSet({
    selectionSet: field.selectionSet,
    parentQuery: baseQuery,
    parentType: currentTypeAsObjectType,
    info: info,
    path: path,
    plugins: plugins,
    argumentResolvers: argumentResolvers
  });
  return baseQuery;
};

var extractQueriesFromSelectionSet = function extractQueriesFromSelectionSet(_ref3) {
  var selectionSet = _ref3.selectionSet,
      path = _ref3.path,
      rest = _objectWithoutPropertiesLoose(_ref3, ["selectionSet", "path"]);

  return selectionSet.selections.reduce(function (reducedQueries, selection) {
    if (selection.kind === 'Field') {
      var _extends2;

      return _extends({}, reducedQueries, (_extends2 = {}, _extends2[getNameOrAlias(selection)] = extractQueriesFromField(_extends({
        field: selection,
        path: [].concat(path, [getNameOrAlias(selection)])
      }, rest)), _extends2));
    } else if (selection.kind === 'InlineFragment') {
      return _extends({}, reducedQueries, {}, extractQueriesFromSelectionSet(_extends({
        selectionSet: selection.selectionSet,
        path: path
      }, rest)));
    } else {
      var fragment = rest.info.fragments[selection.name.value];
      return _extends({}, reducedQueries, {}, extractQueriesFromSelectionSet(_extends({
        selectionSet: fragment.selectionSet,
        path: path
      }, rest)));
    }
  }, {});
};

var directiveTypeDefs = "\nenum AqlEdgeDirection {\n  OUTBOUND\n  INBOUND\n  ANY\n}\n\nenum AqlSortOrder {\n  DESC\n  ASC\n}\n\ndirective @document(\n  collection: String!\n  key: String\n) on FIELD_DEFINITION\n\ndirective @node(\n  edgeCollection: String!\n  direction: AqlEdgeDirection!\n) on FIELD_DEFINITION\n\ndirective @filter(\n  statement: String!\n) on FIELD_DEFINITION\n\ndirective @sort(\n  property: String!\n  order: AqlSortOrder\n) on FIELD_DEFINITION\n\ndirective @limit(\n  skip: String\n  count: String!\n) on FIELD_DEFINITION\n\ndirective @edge(\n  direction: AqlEdgeDirection!\n  collection: String!\n) on FIELD_DEFINITION\n\ndirective @edgeNode on FIELD_DEFINITION\n\ndirective @aql(\n  expression: String!\n) on FIELD_DEFINITION\n\ndirective @subquery(\n  query: String!\n) on FIELD_DEFINITION\n\ndirective @key on FIELD_DEFINITION\n";

export { createResolver, directiveTypeDefs, extractQueriesFromField, defaultPlugins as plugins, resolver };
//# sourceMappingURL=graphql-arangodb.esm.js.map
